package tests;

import datetime.DateCalculations.*;
import datetime.DateTimeCalculations.*;
import datetime.DateTimeTimezoneCalculations.DateTimeTimezone;
import datetime.DateTimeTimezoneCalculations.DateTimeTimezoneReference;
import references.references.*;

import static datetime.DateCalculations.DateCalculations.*;
import static datetime.DateTimeCalculations.DateTimeCalculations.*;
import static datetime.DateTimeTimezoneCalculations.DateTimeTimezoneCalculations.*;
import static references.references.references.*;
import static testing.testing.testing.*;

public class tests {
    public static double test(){
        NumberReference failures;

        failures = CreateNumberReference(0d);

        testIsLeapYear(failures);
        testDayToDate(failures);
        dateToDayTest(failures);
        testSecondsToDateTime(failures);
        testSecondsToDateTime2(failures);
        testDatetimeToSecondsTest(failures);
        testIsValidDate(failures);
        testDayDifferenceBetweenTwoDayes(failures);
        testWeekDay(failures);
        testTimezones1(failures);
        testTimezones2(failures);
        testDaysBetween(failures);
        testToString(failures);

        return failures.numberValue;
    }

    public static void testIsLeapYear(NumberReference failures){
        boolean success;
        StringReference errorMessage;
        BooleanReference isLeapYearReference;
        DateTime aDateTime;
        DateTimeReference dateTimeReference;

        isLeapYearReference = new BooleanReference();
        errorMessage = new StringReference();

        success = IsLeapYearWithCheck(2012d, isLeapYearReference, errorMessage);
        AssertTrue(success, failures);
        AssertTrue(isLeapYearReference.booleanValue, failures);

        success = IsLeapYearWithCheck(2016d, isLeapYearReference, errorMessage);
        AssertTrue(success, failures);
        AssertTrue(isLeapYearReference.booleanValue, failures);

        success = IsLeapYearWithCheck(2000d, isLeapYearReference, errorMessage);
        AssertTrue(success, failures);
        AssertTrue(isLeapYearReference.booleanValue, failures);

        success = IsLeapYearWithCheck(2100d, isLeapYearReference, errorMessage);
        AssertTrue(success, failures);
        AssertFalse(isLeapYearReference.booleanValue, failures);

        success = IsLeapYearWithCheck(2200d, isLeapYearReference, errorMessage);
        AssertTrue(success, failures);
        AssertFalse(isLeapYearReference.booleanValue, failures);

        success = IsLeapYearWithCheck(2018d, isLeapYearReference, errorMessage);
        AssertTrue(success, failures);
        AssertFalse(isLeapYearReference.booleanValue, failures);

        // Excel bug.
        success = IsLeapYearWithCheck(1900d, isLeapYearReference, errorMessage);
        AssertTrue(success, failures);
        AssertFalse(isLeapYearReference.booleanValue, failures);

        // For generating inputs:
        aDateTime = CreateDateTime(2019d, 11d, 28d, 10d, 6d, 40d);
        GetWeeksFromSeconds(GetSecondsFromDateTime(aDateTime));
        GetDateFromDateTime(aDateTime);
        dateTimeReference = new DateTimeReference();
        AddMinutesToDateTime(aDateTime, 900d, dateTimeReference, errorMessage);
        AddHoursToDateTime(aDateTime, 48d, dateTimeReference, errorMessage);
        AddDaysToDateTime(aDateTime, 90d, dateTimeReference, errorMessage);
        AddWeeksToDateTime(aDateTime, 20d, dateTimeReference, errorMessage);
    }

    public static void testDayToDate(NumberReference failures){
        boolean success;
        Date date;
        DateReference dateReference;
        StringReference errorMessage;

        dateReference = new DateReference();
        errorMessage = new StringReference();

        success = DayToDateWithCheck(17483d, dateReference, errorMessage);
        AssertTrue(success, failures);
        date = dateReference.date;
        AssertEquals(2017d, date.year, failures);
        AssertEquals(11d, date.month, failures);
        AssertEquals(13d, date.day, failures);

        success = DayToDateWithCheck(0d, dateReference, errorMessage);
        AssertTrue(success, failures);
        date = dateReference.date;
        AssertEquals(1970d, date.year, failures);
        AssertEquals(1d, date.month, failures);
        AssertEquals(1d, date.day, failures);
    }

    public static void dateToDayTest(NumberReference failures){
        Date date;
        double day;
        NumberReference dayNumberReferenceReference;
        StringReference message;
        boolean success;

        dayNumberReferenceReference = new NumberReference();
        message = new StringReference();

        date = CreateDate(2017d, 11d, 13d);
        success = DateToDaysWithCheck(date, dayNumberReferenceReference, message);
        AssertTrue(success, failures);
        day = dayNumberReferenceReference.numberValue;
        AssertEquals(17483d, day, failures);

        date = CreateDate(1970d, 1d, 1d);
        success = DateToDaysWithCheck(date, dayNumberReferenceReference, message);
        AssertTrue(success, failures);
        day = dayNumberReferenceReference.numberValue;
        AssertEquals(0d, day, failures);

        date = CreateDate(2020d, 2d, 28d);
        success = AddDaysToDate(date, 2d, message);

        AssertTrue(success, failures);

        if(success) {
            AssertEquals(2020d, date.year, failures);
            AssertEquals(3d, date.month, failures);
            AssertEquals(1d, date.day, failures);
        }

        date = CreateDate(2023d, 4d, 30d);
        success = AddDaysToDate(date, 1d, message);

        AssertTrue(success, failures);

        if(success) {
            AssertEquals(2023d, date.year, failures);
            AssertEquals(5d, date.month, failures);
            AssertEquals(1d, date.day, failures);
        }
    }

    public static void testSecondsToDateTime(NumberReference failures){
        double seconds;
        DateTime dateTime;
        DateTimeReference dateTimeReference;
        StringReference errorMessage;
        boolean success;

        dateTimeReference = new DateTimeReference();
        errorMessage = new StringReference();

        seconds = 1510565444d;

        success = GetDateTimeFromSeconds(seconds, dateTimeReference, errorMessage);
        AssertTrue(success, failures);
        dateTime = dateTimeReference.dateTime;
        AssertEquals(2017d, dateTime.date.year, failures);
        AssertEquals(11d, dateTime.date.month, failures);
        AssertEquals(13d, dateTime.date.day, failures);
        AssertEquals(9d, dateTime.hours, failures);
        AssertEquals(30d, dateTime.minutes, failures);
        AssertEquals(44d, dateTime.seconds, failures);
    }

    public static void testDatetimeToSecondsTest(NumberReference failures){
        DateTime dateTime;
        double seconds;

        dateTime = CreateDateTime(2017d, 11d, 13d, 9d, 30d, 44d);

        seconds = GetSecondsFromDateTime(dateTime);

        AssertEquals(1510565444d, seconds, failures);
    }

    public static void testIsValidDate(NumberReference failures){
        Date date;

        date = CreateDate(2017d, 11d, 13d);
        AssertTrue(IsValidDate(date, new StringReference()), failures);

        date = CreateDate(2017d, 2d, 31d);
        AssertFalse(IsValidDate(date, new StringReference()), failures);

        date = CreateDate(2016d, 2d, 29d);
        AssertTrue(IsValidDate(date, new StringReference()), failures);

        date = CreateDate(2017d, 2d, 29d);
        AssertFalse(IsValidDate(date, new StringReference()), failures);

        date = CreateDate(2017d, 0d, 29d);
        AssertFalse(IsValidDate(date, new StringReference()), failures);

        date = CreateDate(2017d, 1d, 32d);
        AssertFalse(IsValidDate(date, new StringReference()), failures);
    }

    public static void testDayDifferenceBetweenTwoDayes(NumberReference failures){
        Date date1, date2;
        double day1, day2, days;
        NumberReference dayNumberReferenceReference;
        StringReference errorMessage;
        boolean success;

        dayNumberReferenceReference = new NumberReference();
        errorMessage = new StringReference();

        date1 = CreateDate(2014d, 3d, 5d);
        date2 = CreateDate(2018d, 9d, 23d);

        success = DateToDaysWithCheck(date1, dayNumberReferenceReference, errorMessage);
        AssertTrue(success, failures);
        day1 = dayNumberReferenceReference.numberValue;
        success = DateToDaysWithCheck(date2, dayNumberReferenceReference, errorMessage);
        AssertTrue(success, failures);
        day2 = dayNumberReferenceReference.numberValue;

        days = day2 - day1;

        AssertEquals(days, 1663d, failures);
    }

    public static void testWeekDay(NumberReference failures){
        Date date1, date2;
        double day1, day2;
        NumberReference weekDayNumberReference;
        StringReference errorMessage;
        boolean success;

        weekDayNumberReference = new NumberReference();
        errorMessage = new StringReference();

        date1 = CreateDate(2014d, 3d, 5d);
        date2 = CreateDate(2018d, 9d, 23d);

        success = DateToWeekdayNumberWithCheck(date1, weekDayNumberReference, errorMessage);
        AssertTrue(success, failures);
        day1 = weekDayNumberReference.numberValue;
        success = DateToWeekdayNumberWithCheck(date2, weekDayNumberReference, errorMessage);
        AssertTrue(success, failures);
        day2 = weekDayNumberReference.numberValue;

        AssertEquals(day1, 3d, failures);
        AssertEquals(day2, 7d, failures);
    }

    public static void testDaysBetween(NumberReference failures){
        Date a, b;
        double days;

        a = CreateDate(2014d, 3d, 5d);
        b = CreateDate(2018d, 9d, 23d);

        days = DaysBetweenDates(a, b);

        AssertEquals(1663d, days, failures);
    }

    public static void testTimezones1(NumberReference failures){
        StringReference errorMessage;
        boolean success;
        DateTimeTimezoneReference dateTimeTimezoneReference;
        DateTimeTimezone dateTimeTimezone;
        DateTime dateTime;

        dateTime = CreateDateTime(2018d, 9d, 24d, 7d, 20d, 0d);

        dateTimeTimezoneReference = new DateTimeTimezoneReference();
        errorMessage = new StringReference();

        success = CreateDateTimeTimezoneFromDateTimeAndTimeZoneInHoursAndMinutes(dateTime, 2d, 0d, dateTimeTimezoneReference, errorMessage);
        AssertTrue(success, failures);
        dateTimeTimezone = dateTimeTimezoneReference.dateTimeTimezone;

        AssertEquals(2018d, dateTimeTimezone.dateTime.date.year, failures);
        AssertEquals(9d, dateTimeTimezone.dateTime.date.month, failures);
        AssertEquals(24d, dateTimeTimezone.dateTime.date.day, failures);
        AssertEquals(9d, dateTimeTimezone.dateTime.hours, failures);
        AssertEquals(20d, dateTimeTimezone.dateTime.minutes, failures);
        AssertEquals(0d, dateTimeTimezone.dateTime.seconds, failures);
    }

    public static void testTimezones2(NumberReference failures){
        StringReference errorMessage;
        boolean success;
        DateTimeTimezone dateTimeTimezone;
        DateTimeReference dateTimeReference;

        errorMessage = new StringReference();

        dateTimeTimezone = CreateDateTimeTimezoneInHoursAndMinutes(2018d, 9d, 24d, 9d, 20d, 0d, 2d, 0d);
        dateTimeReference = new DateTimeReference();
        success = GetDateFromDateTimeTimeZone(dateTimeTimezone, dateTimeReference, errorMessage);
        AssertTrue(success, failures);

        AssertEquals(2018d, dateTimeReference.dateTime.date.year, failures);
        AssertEquals(9d, dateTimeReference.dateTime.date.month, failures);
        AssertEquals(24d, dateTimeReference.dateTime.date.day, failures);
        AssertEquals(7d, dateTimeReference.dateTime.hours, failures);
        AssertEquals(20d, dateTimeReference.dateTime.minutes, failures);
        AssertEquals(0d, dateTimeReference.dateTime.seconds, failures);
    }

    public static void testSecondsToDateTime2(NumberReference failures){
        double seconds;
        DateTime dateTime;
        DateTimeReference dateTimeReference;
        StringReference errorMessage;
        boolean success;

        dateTimeReference = new DateTimeReference();
        errorMessage = new StringReference();

        seconds = 1601596487;

        success = GetDateTimeFromSeconds(seconds, dateTimeReference, errorMessage);
        AssertTrue(success, failures);
        dateTime = dateTimeReference.dateTime;
        AssertEquals(2020d, dateTime.date.year, failures);
        AssertEquals(10d, dateTime.date.month, failures);
        AssertEquals(1d, dateTime.date.day, failures);
        AssertEquals(23d, dateTime.hours, failures);
        AssertEquals(54d, dateTime.minutes, failures);
        AssertEquals(47d, dateTime.seconds, failures);
    }

    public static void testToString(NumberReference failures) {
        char [] str;
        Date date, date2;
        DateTime dateTime, dateTime2;

        date = CreateDate(2020d, 9d, 14d);

        str = DateToStringISO8601(date);

        AssertStringEquals(str, "2020-09-14".toCharArray(), failures);

        date2 = DateFromStringISO8601(str);

        dateTime = CreateDateTime(2020d, 9d, 14d, 13d, 59d, 44d);

        str = DateTimeToStringISO8601(dateTime);

        AssertTrue(DateEquals(date, date2), failures);

        AssertStringEquals(str, "2020-09-14T13:59:44".toCharArray(), failures);

        AssertEquals(date.year, date2.year, failures);
        AssertEquals(date.month, date2.month, failures);
        AssertEquals(date.day, date2.day, failures);

        dateTime2 = DateTimeFromStringISO8601(str);

        AssertTrue(DateTimeEquals(dateTime, dateTime2), failures);
    }
}
